//
//  Copyright 2011 (c) Finnbarr P. Murphy.  All rights reserved.
//

const St = imports.gi.St;
const Main = imports.ui.main;
const Shell = imports.gi.Shell;
const Lang = imports.lang;
const Clutter = imports.gi.Clutter;
const Mainloop = imports.mainloop;
const Tweener = imports.ui.tweener;

const HOT_CORNER_ACTIVATION_TIMEOUT = 0.5;

const SHOW_RIPPLE = true;      // change to false if you do not want the ripple



function RightHotCorner() {
    this._init.apply(this, arguments);
}

RightHotCorner.prototype = {
    _init : function(show_ripple) {
        this._entered = false;
        this._activationTime = 0;
        this._show_ripple = show_ripple;

        this._button = new St.Button({ name: 'rhc-hotcorner-button',
                                       style_class: 'rhc-button',
                                       reactive: true,
                                       can_focus: true });
        this._button._delegate = this._button;
        Main.panel._rightBox.add_actor(this._button);

        let monitor = global.get_primary_monitor();
        let cornerX = monitor.x + monitor.width;
        let cornerY = monitor.y;

        this.actor = new Clutter.Group({ width: 5,
                                         height: 5,
                                         reactive: true });
        this._corner = new Clutter.Rectangle({ width: 5,
                                               height: 5,
                                               opacity: 0,
                                               reactive: true });

        this.actor.add_actor(this._corner);
        this.actor.set_position(cornerX, cornerY);
        this.actor.set_anchor_point_from_gravity(Clutter.Gravity.NORTH_EAST);

        this.actor.connect('enter-event',
                           Lang.bind(this, this._onEnvironsEntered));
        this.actor.connect('leave-event',
                           Lang.bind(this, this._onEnvironsRight));
        this.actor.connect('button-release-event',
                           Lang.bind(this, this._onCornerClicked));

        this._corner.connect('enter-event',
                             Lang.bind(this, this._onCornerEntered));
        this._corner.connect('button-release-event',
                             Lang.bind(this, this._onCornerClicked));
        this._corner.connect('leave-event',
                             Lang.bind(this, this._onCornerRight));

        this._corner._delegate = this._corner;

        Main.chrome.addActor(this.actor, { visibleInOverview: true, affectsStruts: false });
    },

    destroy: function() {
        this.actor.destroy();
    },

    _addRipple : function(delay, time, startScale, startOpacity, finalScale, finalOpacity) {
        let [x, y] = this._corner.get_transformed_position();
        let rhripple = new St.BoxLayout({ style_class: 'rhc-ripple-box',
                                          opacity: 255 * Math.sqrt(startOpacity),
                                          scale_x: startScale,
                                          scale_y: startScale,
                                          x: x,
                                          y: y });
        rhripple._opacity =  startOpacity;
        rhripple.set_anchor_point_from_gravity(Clutter.Gravity.NORTH_EAST);
        Tweener.addTween(rhripple, { _opacity: finalOpacity,
                                     scale_x: finalScale,
                                     scale_y: finalScale,
                                     delay: delay,
                                     time: time,
                                     transition: 'linear',
                                     onUpdate: function() { rhripple.opacity = 255 * Math.sqrt(rhripple._opacity); },
                                     onComplete: function() { rhripple.destroy(); } });
        Main.uiGroup.add_actor(rhripple);
    },

    _rippleAnimation: function() {
        if (this._show_ripple) {
           this._addRipple(0.0,   0.83,  0.25,  1.0,    1.5,  0.0);
           this._addRipple(0.05,  1.0,   0.0,   0.7,    1.25, 0.0);
           this._addRipple(0.35,  1.0,   0.0,   0.3,    1,    0.0);
        }
    },

    _onEnvironsEntered : function() {
        if (this._button)
            this._button.hover = true;
    },

    _onCornerEntered : function() {
        if (!this._entered) {
            this._entered = true;
            if (!Main.overview.animationInProgress) {
                this._activationTime = Date.now() / 1000;
                this._rippleAnimation();
                Main.overview.toggle();
            }
        }
        return false;
    },

    _onCornerClicked : function() {
         if (!Main.overview.animationInProgress)
             this._maybeToggleOverviewOnClick();
         return true;
    },

    _onCornerRight : function(actor, event) {
        if (event.get_related() != this.actor)
            this._entered = false;
        return true;
    },

    _onEnvironsRight : function(actor, event) {
        if (this._button)
            this._button.hover = false;
        if (event.get_related() != this._corner)
            this._entered = false;
        return false;
    },

    _maybeToggleOverviewOnClick: function() {
        if (this._activationTime == 0 || 
            Date.now() / 1000 - this._activationTime > HOT_CORNER_ACTIVATION_TIMEOUT)
            Main.overview.toggle();
        this._activationTime = 0;
    }
};


function main() {
    new RightHotCorner(SHOW_RIPPLE);
}
